// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com


package engine.net.client.msg.login;


import engine.net.AbstractConnection;
import engine.net.ByteBufferReader;
import engine.net.ByteBufferWriter;
import engine.net.client.Protocol;
import engine.net.client.msg.ClientNetMsg;

public class ServerStatusMsg extends ClientNetMsg {

    private int serverID;
    private byte isUp;

    /**
     * This is the general purpose constructor.
     */
    public ServerStatusMsg(int serverID, byte isUp) {
        super(Protocol.ARCSERVERSTATUS);
        this.serverID = serverID;
        this.isUp = isUp;
    }

    /**
     * This constructor is used by NetMsgFactory. It attempts to deserialize the
     * ByteBuffer into a message. If a BufferUnderflow occurs (based on reading
     * past the limit) then this constructor Throws that Exception to the
     * caller.
     */
    public ServerStatusMsg(AbstractConnection origin, ByteBufferReader reader) {
        super(Protocol.ARCSERVERSTATUS, origin, reader);
    }

    /**
     * Serializes the subclass specific items to the supplied NetMsgWriter.
     */
    @Override
    protected void _serialize(ByteBufferWriter writer) {
        writer.putInt(this.serverID);
        writer.put(this.isUp);
    }

    /**
     * Deserializes the subclass specific items from the supplied NetMsgReader.
     */
    @Override
    protected void _deserialize(ByteBufferReader reader) {
        this.serverID = reader.getInt();
        this.isUp = reader.get();
    }

    public int getServerID() {
        return this.serverID;
    }

    public void setServerID(int value) {
        this.serverID = value;
    }

    public byte getIsUp() {
        return this.isUp;
    }

    public void setIsUp(byte value) {
        this.isUp = value;
    }

    public boolean isUp() {
        return (this.isUp == 0x01);
    }
}
